@@ -1,24 +1,20 @@
-require 'rabbit/gtk'
+# Copyright (C) 2005-2024  Sutou Kouhei <kou@cozmixng.org>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-module Gtk
-  class Action
-    alias _activate activate
-    def activate(&block)
-      @block = block
-      _activate
-    ensure
-      @block = nil
-    end
-
-    def block_given?
-      not @block.nil?
-    end
-
-    def call(*args, &block)
-      @block.call(*args, &block)
-    end
-  end
-end
+require "rabbit/gtk"
 
 require 'rabbit/rabbit'
 require 'rabbit/stock'
@@ -129,9 +125,6 @@ module Rabbit
       end
     end
 
-    dir = ::File.join("rabbit", "action")
-    require_files_under_directory_in_load_path(dir)
-
     module_function
     def update_processing_action_status(canvas)
       canvas.action("ToggleIndexMode").sensitive = !canvas.processing?
@@ -141,3 +134,7 @@ module Rabbit
     end
   end
 end
+
+require "rabbit/action/basic"
+require "rabbit/action/radio"
+require "rabbit/action/toggle"
