@@ -1,10 +1,26 @@
+# Copyright (C) 2004-2024  Sutou Kouhei <kou@cozmixng.org>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+
 require "forwardable"
 require "rabbit/gtk"
 
 require "rabbit/rabbit"
-require 'rabbit/frame'
-require 'rabbit/renderer'
-require 'rabbit/element'
+require "rabbit/filename"
+require "rabbit/frame"
+require "rabbit/element"
 require "rabbit/parser"
 require "rabbit/theme/manager"
 require "rabbit/front"
@@ -18,7 +34,7 @@ module Rabbit
 
     include GetText
 
-    INTERNAL_DPI = 80.0
+    INTERNAL_DPI = 96.0
 
     def_delegators(:@frame, :icon, :icon=, :set_icon)
     def_delegators(:@frame, :icon_list, :icon_list=, :set_icon_list)
@@ -27,9 +43,16 @@ module Rabbit
     def_delegators(:@frame, :iconify, :window)
     def_delegators(:@frame, :fullscreen_available?)
     def_delegators(:@frame, :iconify_available?)
+    def_delegators(:@frame, :toggle_terminal)
+    def_delegators(:@frame, :in_terminal?)
 
     def_delegators(:@renderer, :width, :height)
-    def_delegators(:@renderer, :width=, :height=)
+    # Deprecated. Use #base_width= instead.
+    def_delegator(:@renderer, :base_width=, :width=)
+    # Deprecated. Use #base_height= instead.
+    def_delegator(:@renderer, :base_height=, :height=)
+    def_delegators(:@renderer, :base_width, :base_height)
+    def_delegators(:@renderer, :base_width=, :base_height=)
     def_delegators(:@renderer, :paper_width, :paper_height)
     def_delegators(:@renderer, :paper_width=, :paper_height=)
     def_delegators(:@renderer, :margin_left, :margin_left=)
@@ -60,6 +83,8 @@ module Rabbit
     def_delegators(:@renderer, :shear_context)
     def_delegators(:@renderer, :save_context, :restore_context)
     def_delegators(:@renderer, :draw_line, :draw_rectangle, :draw_arc)
+    def_delegators(:@renderer, :draw_lines)
+    def_delegators(:@renderer, :draw_polygon)
     def_delegators(:@renderer, :draw_circle, :draw_layout, :draw_pixbuf)
     def_delegators(:@renderer, :draw_arc_by_radius, :draw_circle_by_radius)
     def_delegators(:@renderer, :draw_slide, :draw_polygon, :draw_flag)
@@ -68,24 +93,15 @@ module Rabbit
     def_delegators(:@renderer, :draw_poppler_page)
     def_delegators(:@renderer, :draw_link)
 
-    def_delegators(:@renderer, :draw_cube, :draw_sphere, :draw_cone)
-    def_delegators(:@renderer, :draw_torus, :draw_tetrahedron)
-    def_delegators(:@renderer, :draw_octahedron, :draw_dodecahedron)
-    def_delegators(:@renderer, :draw_icosahedron, :draw_teapot)
-
-    def_delegators(:@renderer, :gl_compile, :gl_call_list)
-    def_delegators(:@renderer, :new_list_id)
-
     def_delegators(:@renderer, :z_far, :z_view)
 
-    def_delegators(:@renderer, :to_attrs, :flag_size)
+    def_delegators(:@renderer, :flag_size)
 
     def_delegators(:@renderer, :create_pango_context, :pango_context=)
 
     def_delegators(:@renderer, :confirm)
 
     def_delegators(:@renderer, :display?, :printable?)
-    def_delegators(:@renderer, :x_dpi, :y_dpi)
 
     def_delegators(:@renderer, :whiteouting?, :blackouting?)
     def_delegators(:@renderer, :toggle_whiteout, :toggle_blackout)
@@ -322,22 +338,26 @@ module Rabbit
       apply_theme(@theme_name, &block)
     end
 
-    def parse(source=nil, callback=nil, &block)
+    def parse(source=nil, progress=nil, &block)
       id = Object.new.__id__
       @parse_request_queue.push(id)
       @source = source || @source
       begin
         index = current_index
+        current_allotted_time = allotted_time
         keep_index do
           @renderer.pre_parse
           clear
-          Parser.parse(self, @source)
+          Parser.parse(self, @source, progress: progress)
+          new_allotted_time = allotted_time
+          reset_timer if new_allotted_time != current_allotted_time
+          apply_timer
           set_current_index(index)
           reload_theme do
             if @parse_request_queue.last != id
               raise ParseFinish
             end
-            callback.call if callback
+            progress.call if progress
           end
           @renderer.post_parse
           index = current_index
@@ -355,9 +375,9 @@ module Rabbit
       end
     end
 
-    def reload_source(callback=nil, &block)
+    def reload_source(progress=nil, &block)
       if need_reload_source?
-        parse(nil, callback, &block)
+        parse(nil, progress, &block)
       end
     end
 
@@ -411,11 +431,19 @@ module Rabbit
       @renderer.post_iconify
     end
 
+    def pre_terminal
+      @renderer.pre_terminal
+    end
+
+    def post_terminal
+      @renderer.post_terminal
+    end
+
     def saved_image_base_name
       if @saved_image_base_name
-        base_name = GLib.filename_to_utf8(@saved_image_base_name)
+        base_name = @saved_image_base_name.encode("UTF-8")
       else
-        base_name = title.dup
+        base_name = Filename.sanitize(title)
       end
       base_name << "-index" if index_mode?
       base_name
@@ -518,7 +546,9 @@ module Rabbit
     end
 
     def have_previous?
-      have_previous_slide? or (current_slide and !current_slide.first?)
+      return true if have_previous_slide?
+      return !current_slide.first? if current_slide
+      false
     end
 
     def have_next_slide?
@@ -526,7 +556,9 @@ module Rabbit
     end
 
     def have_next?
-      have_next_slide? or (current_slide and !current_slide.last?)
+      return true if have_next_slide?
+      return !current_slide.last? if current_slide
+      false
     end
 
     def cache_all_slides
@@ -543,10 +575,11 @@ module Rabbit
       stop_auto_redraw_timer
       timer = GLib::Timeout.add(interval * 1000) do
         if quitted?
-          false
+          @auto_redraw_timer = nil
+          GLib::Source::REMOVE
         else
           activate("Redraw")
-          true
+          GLib::Source::CONTINUE
         end
       end
       @auto_redraw_timer = timer
@@ -617,7 +650,18 @@ module Rabbit
 
     def allotted_time
       time = @allotted_time
-      time ||= title_slide.allotted_time if title_slide
+      if time.nil? and title_slide
+        time = title_slide.allotted_time
+        if time.nil?
+          start_time = title_slide.start_time
+          end_time = title_slide.end_time
+          if start_time and end_time
+            start_time = Time.parse(start_time)
+            end_time = Time.parse(end_time)
+            time = end_time - start_time
+          end
+        end
+      end
       Utils.ensure_time(time)
     end
 
@@ -631,6 +675,23 @@ module Rabbit
 
     def reset_timer
       @limit_time = nil
+      reload_theme
+    end
+
+    def apply_timer
+      return unless title_slide
+
+      start_time = title_slide.start_time
+      end_time = title_slide.end_time
+      return if start_time.nil?
+      return if end_time.nil?
+
+      start_time = Time.parse(start_time)
+      end_time = Time.parse(end_time)
+      return unless (start_time..end_time).cover?(Time.now)
+
+      @allotted_time = end_time - start_time
+      @limit_time = end_time
     end
 
     def font_resolution
@@ -688,7 +749,6 @@ module Rabbit
 
     def clear
       clear_comments
-      reset_timer
       stop_auto_redraw_timer
       clear_slides
       clear_index_slides
