@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2013  Kouhei Sutou <kou@cozmixng.org>
+# Copyright (C) 2012-2021  Sutou Kouhei <kou@cozmixng.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,10 +16,10 @@
 
 require "yaml"
 
-require "rabbit/console"
 require "rabbit/author-configuration"
-require "rabbit/slide-configuration"
+require "rabbit/console"
 require "rabbit/path-manipulatable"
+require "rabbit/slide-configuration"
 require "rabbit/source-generator"
 
 module Rabbit
@@ -34,17 +34,45 @@ module Rabbit
         end
       end
 
+      class Data < Struct.new(:title,
+                              :allotted_time,
+                              :slide_conf,
+                              :author_conf)
+        def available_markup_languages
+          {
+            :markdown => "Markdown",
+            :rd => "RD",
+            :hiki => "Hiki",
+            :pdf => "PDF",
+          }
+        end
+
+        def default_markup_language
+          :rd
+        end
+
+        def markup_language
+          author_conf.markup_language || default_markup_language
+        end
+
+        def save
+          author_conf.save
+        end
+      end
+
       def initialize
-        @title = nil
-        @allotted_time = nil
-        @slide_conf = nil
-        @author_conf = nil
+        @use_gui = true
+        @data = Data.new
         @logger = nil
       end
 
       def run(arguments)
         parse_command_line_arguments(arguments)
 
+        if @use_gui
+          return false unless show_gui
+        end
+
         validate
         unless @validation_errors.empty?
           messages = (@validation_errors + [_("See --help for example")])
@@ -53,7 +81,7 @@ module Rabbit
         end
 
         run_command
-        @author_conf.save
+        @data.save
         true
       end
 
@@ -68,16 +96,18 @@ module Rabbit
       def setup_options(parser, options)
         @options = options
         @logger = @options.default_logger
-        @author_conf = AuthorConfiguration.new(@logger)
-        @author_conf.load
-        @slide_conf = SlideConfiguration.new(@logger)
-        @slide_conf.author = @author_conf
+        @data.author_conf = AuthorConfiguration.new(@logger)
+        @data.author_conf.load
+        @data.slide_conf = SlideConfiguration.new(@logger)
+        @data.slide_conf.author = @data.author_conf
 
         format = _("Usage: %s COMMAND [OPTIONS]\n" \
                    " e.g.: %s new \\\n" \
                    "          --id rubykaigi2012 \\\n" \
                    "          --base-name rabbit-introduction \\\n" \
                    "          --markup-language rd \\\n" \
+                   "          --width 800 \\\n" \
+                   "          --height 450 \\\n" \
                    "          --name \"Kouhei Sutou\" \\\n" \
                    "          --email kou@cozmixng.org \\\n" \
                    "          --rubygems-user kou \\\n" \
@@ -92,6 +122,13 @@ module Rabbit
         parser.separator(_("  new:    create a new slide"))
         parser.separator(_("  change: change an existing slide"))
 
+        parser.separator("")
+        parser.separator(_("User interface"))
+        parser.on("--no-use-gui",
+                  _("Don't use GUI")) do |boolean|
+          @use_gui = boolean
+        end
+
         parser.separator("")
         parser.separator(_("Slide information"))
 
@@ -99,7 +136,7 @@ module Rabbit
                   _("Slide ID"),
                   _("(e.g.: %s)") % "--id=rubykaigi2012",
                   _("(must)")) do |id|
-          @slide_conf.id = id
+          @data.slide_conf.id = id
         end
 
         messages = [
@@ -109,30 +146,38 @@ module Rabbit
         ]
         parser.on("--base-name=NAME",
                   *messages) do |base_name|
-          @slide_conf.base_name = base_name
+          @data.slide_conf.base_name = base_name
         end
 
-        available_markup_languages = [:rd, :hiki, :markdown]
-        label = "[" + available_markup_languages.join(", ") + "]"
+        available_markup_languages = @data.available_markup_languages
+        label = "[" + available_markup_languages.keys.join(", ") + "]"
         messages = [
           _("Markup language for the new slide"),
           _("(e.g.: %s)") % "--markup-language=rd",
           _("(available markup languages: %s)") % label,
         ]
-        if @author_conf.markup_language
-          messages << _("(default: %s)") % @author_conf.markup_language
+        if @data.author_conf.markup_language
+          messages << _("(default: %s)") % @data.author_conf.markup_language
         end
         messages << _("(optional)")
-        parser.on("--markup-language=LANGUAGE", available_markup_languages,
+        parser.on("--markup-language=LANGUAGE", available_markup_languages.keys,
                   *messages) do |language|
-          @author_conf.markup_language = language
+          @data.author_conf.markup_language = language
         end
 
         parser.on("--title=TITLE",
                   _("Title of the new slide"),
                   _("(e.g.: %s)") % _("--title=\"Rabbit Introduction\""),
                   _("(optional)")) do |title|
-          @title = title
+          @data.title = title
+        end
+
+        parser.on("--licenses=LICENSE,LICENSE,...",
+                  Array,
+                  _("License of the new slide"),
+                  _("(e.g.: %s)") % "--licenses=CC-BY-SA-4.0,GFDL-1.3-or-later",
+                  _("(optional)")) do |licenses|
+          @data.slide_conf.licenses.concat(licenses)
         end
 
         parser.on("--tags=TAG,TAG,...",
@@ -140,21 +185,55 @@ module Rabbit
                   _("Tags of the new slide"),
                   _("(e.g.: %s)") % "--tags=rabbit,presentation,ruby",
                _("(optional)")) do |tags|
-          @slide_conf.tags.concat(tags)
+          @data.slide_conf.tags.concat(tags)
         end
 
         parser.on("--allotted-time=TIME",
-                  _("Allotted time in presentaion"),
+                  _("Allotted time in presentation"),
                   _("(e.g.: %s)") % "--allotted-time=5m",
                   _("(optional)")) do |allotted_time|
-          @allotted_time = allotted_time
+          @data.allotted_time = allotted_time
         end
 
-        parser.on("--presentation-date=DATE",
+        parser.on("--presentation-date=DATE", Date,
                   _("Presentation date with the new slide"),
-                  _("(e.g.: %s)") % "--presentation-date=2012/06/29",
+                  _("(e.g.: %s)") % "--presentation-date=2012-06-29",
                   _("(optional)")) do |date|
-          @slide_conf.presentation_date = date
+          @data.slide_conf.presentation_date = date
+        end
+
+        presentation_start_time_example =
+          "--presentation-start-time=2012-06-29T10:30:00+0900"
+        parser.on("--presentation-start-time=TIME", Time,
+                  _("Presentation start time"),
+                  _("(e.g.: %s)") % presentation_start_time_example,
+                  _("(optional)")) do |time|
+          @data.slide_conf.presentation_start_time = time
+        end
+
+        presentation_end_time_example =
+          "--presentation-end-time=2012-06-29T11:00:00+0900"
+        parser.on("--presentation-end-time=TIME", Time,
+                  _("Presentation end time"),
+                  _("(e.g.: %s)") % presentation_end_time_example,
+                  _("(optional)")) do |time|
+          @data.slide_conf.presentation_end_time = time
+        end
+
+        width_example = "--width=800"
+        parser.on("--width=WIDTH", Integer,
+                  _("The default slide width"),
+                  _("(e.g.: %s)") % width_example,
+                  _("(optional)")) do |width|
+          @data.slide_conf.width = width
+        end
+
+        height_example = "--width=450"
+        parser.on("--height=HEIGHT", Integer,
+                  _("The default slide height"),
+                  _("(e.g.: %s)") % height_example,
+                  _("(optional)")) do |height|
+          @data.slide_conf.height = height
         end
 
         parser.separator(_("Your information"))
@@ -163,26 +242,26 @@ module Rabbit
           _("Author name of the new slide"),
           _("(e.g.: %s)") % "--name=\"Kouhei Sutou\"",
         ]
-        if @author_conf.name
-          messages << _("(default: %s)") % @author_conf.name
+        if @data.author_conf.name
+          messages << _("(default: %s)") % @data.author_conf.name
         end
         messages << _("(optional)")
         parser.on("--name=NAME",
                   *messages) do |name|
-          @author_conf.name = name
+          @data.author_conf.name = name
         end
 
         messages = [
           _("Author e-mail of the new slide"),
           _("(e.g.: %s)") % "--email=kou@cozmixng.org",
         ]
-        if @author_conf.email
-          messages << _("(default: %s)") % @author_conf.email
+        if @data.author_conf.email
+          messages << _("(default: %s)") % @data.author_conf.email
         end
         messages << _("(optional)")
         parser.on("--email=EMAIL",
                   *messages) do |email|
-          @author_conf.email = email
+          @data.author_conf.email = email
         end
 
         messages = [
@@ -190,13 +269,13 @@ module Rabbit
           _("It is used to publish your slide to %s") % "RubyGems.org",
           _("(e.g.: %s)") % "--rubygems-user=kou",
         ]
-        if @author_conf.rubygems_user
-          messages << _("(default: %s)") % @author_conf.rubygems_user
+        if @data.author_conf.rubygems_user
+          messages << _("(default: %s)") % @data.author_conf.rubygems_user
         end
         messages << _("(optional)")
         parser.on("--rubygems-user=USER",
                   *messages) do |user|
-          @author_conf.rubygems_user = user
+          @data.author_conf.rubygems_user = user
         end
 
         messages = [
@@ -204,13 +283,13 @@ module Rabbit
           _("It is used to publish your slide to %s") % "SlideShare",
           _("(e.g.: %s)") % "--slideshare-user=kou",
         ]
-        if @author_conf.slideshare_user
-          messages << _("(default: %s)") % @author_conf.slideshare_user
+        if @data.author_conf.slideshare_user
+          messages << _("(default: %s)") % @data.author_conf.slideshare_user
         end
         messages << _("(optional)")
         parser.on("--slideshare-user=USER",
                   *messages) do |user|
-          @author_conf.slideshare_user = user
+          @data.author_conf.slideshare_user = user
         end
 
         messages = [
@@ -218,13 +297,13 @@ module Rabbit
           _("It is used to publish your slide to %s") % "Speaker Deck",
           _("(e.g.: %s)") % "--speaker-deck-user=kou",
         ]
-        if @author_conf.speaker_deck_user
-          messages << _("(default: %s)") % @author_conf.speaker_deck_user
+        if @data.author_conf.speaker_deck_user
+          messages << _("(default: %s)") % @data.author_conf.speaker_deck_user
         end
         messages << _("(optional)")
         parser.on("--speaker-deck-user=USER",
                   *messages) do |user|
-          @author_conf.speaker_deck_user = user
+          @data.author_conf.speaker_deck_user = user
         end
       end
 
@@ -240,6 +319,147 @@ module Rabbit
         ["new", "change"]
       end
 
+      class TextMapper
+        def initialize(data)
+          @data = data
+        end
+
+        def attach(entry)
+          entry.signal_connect(:notify) do |_widget, param_spec|
+            if param_spec.name == "text"
+              if valid?(_widget.text)
+                _widget.style_context.remove_class(Gtk::STYLE_CLASS_ERROR)
+              else
+                _widget.style_context.add_class(Gtk::STYLE_CLASS_ERROR)
+              end
+            end
+          end
+          entry.text = value if value
+        end
+
+        def apply(entry)
+          apply_value(entry.text)
+        end
+      end
+
+      class IntegerMapper
+        def initialize(data)
+          @data = data
+        end
+
+        def attach(entry)
+          entry.value = value if value
+        end
+
+        def apply(entry)
+          apply_value(entry.value_as_int)
+        end
+      end
+
+      class SlideIDMapper < TextMapper
+        private
+        def valid?(value)
+          not value.empty?
+        end
+
+        def value
+          @data.slide_conf.id
+        end
+
+        def apply_value(value)
+          @data.slide_conf.id = value
+        end
+      end
+
+      class SlideBaseNameMapper < TextMapper
+        private
+        def valid?(value)
+          not value.empty?
+        end
+
+        def value
+          @data.slide_conf.base_name
+        end
+
+        def apply_value(value)
+          @data.slide_conf.base_name = value
+        end
+      end
+
+      class SlideMarkupLanguageMapper
+        def initialize(data)
+          @data = data
+        end
+
+        def attach(combo_box)
+          combo_box = combo_box
+          @data.available_markup_languages.each do |key, value|
+            combo_box.append(key.to_s, value)
+          end
+          combo_box.active_id = @data.author_conf.markup_language
+        end
+
+        def apply(combo_box)
+          id = combo_box.active_id
+          id = id.to_sym if id
+          @data.author_conf.markup_language = id
+        end
+      end
+
+      class SlideWidthMapper < IntegerMapper
+        private
+        def value
+          @data.slide_conf.width
+        end
+
+        def apply_value(value)
+          @data.slide_conf.width = value
+        end
+      end
+
+      class SlideHeightMapper < IntegerMapper
+        private
+        def value
+          @data.slide_conf.height
+        end
+
+        def apply_value(value)
+          @data.slide_conf.height = value
+        end
+      end
+
+      def build_gui_mappers
+        {
+          "slide-id" => SlideIDMapper.new(@data),
+          "slide-base-name" => SlideBaseNameMapper.new(@data),
+          "slide-markup-language" => SlideMarkupLanguageMapper.new(@data),
+          "slide-width" => SlideWidthMapper.new(@data),
+          "slide-height" => SlideHeightMapper.new(@data),
+        }
+      end
+
+      def show_gui
+        require "rabbit/gtk"
+
+        mappers = build_gui_mappers
+
+        builder = Gtk::Builder.new(path: File.join(__dir__, "rabbit-slide.ui"))
+        mappers.each do |id, mapper|
+          mapper.attach(builder[id])
+        end
+
+        dialog = builder["dialog"]
+        case dialog.run
+        when Gtk::ResponseType::CANCEL, Gtk::ResponseType::DELETE_EVENT
+          false
+        else
+          mappers.each do |id, mapper|
+            mapper.apply(builder[id])
+          end
+          true
+        end
+      end
+
       def validate
         @validation_errors = []
         validate_command
@@ -260,13 +480,13 @@ module Rabbit
       end
 
       def validate_id
-        if @slide_conf.id.nil?
+        if @data.slide_conf.id.nil?
           @validation_errors << (_("%s is missing") % "--id")
         end
       end
 
       def validate_base_name
-        if @slide_conf.base_name.nil?
+        if @data.slide_conf.base_name.nil?
           @validation_errors << (_("%s is missing") % "--base-name")
         end
       end
@@ -288,9 +508,9 @@ module Rabbit
       def merge_config_yaml
         existing_slide_conf = SlideConfiguration.new(@logger)
         existing_slide_conf.load
-        existing_slide_conf.merge!(@slide_conf.to_hash)
-        @slide_conf = existing_slide_conf
-        @author_conf = @slide_conf.author
+        existing_slide_conf.merge!(@data.slide_conf.to_hash)
+        @data.slide_conf = existing_slide_conf
+        @data.author_conf = @data.slide_conf.author
       end
 
       def generate_directory
@@ -320,8 +540,10 @@ EOD
       def generate_dot_rabbit
         create_file(".rabbit") do |dot_rabbit|
           options = []
-          if @author_conf.markup_language.nil? and @allotted_time
-            options << "--allotted-time #{@allotted_time}"
+          size = [@data.slide_conf.width, @data.slide_conf.height].join(",")
+          options << "--size #{size}"
+          if @data.author_conf.markup_language.nil? and @data.allotted_time
+            options << "--allotted-time #{@data.allotted_time}"
           end
           options << slide_path
           dot_rabbit.puts(options.join("\n"))
@@ -329,7 +551,7 @@ EOD
       end
 
       def generate_slide_configuration
-        @slide_conf.save(base_directory)
+        @data.slide_conf.save(base_directory)
       end
 
       def generate_readme
@@ -339,11 +561,11 @@ EOD
       end
 
       def readme_content
-        markup_language = @author_conf.markup_language || :rd
+        markup_language = @data.markup_language
         generator = Rabbit::SourceGenerator.find(markup_language)
 
         content = ""
-        title = @title || _("TODO: SLIDE TITLE")
+        title = @data.title || _("TODO: SLIDE TITLE")
         content << generator.heading(1, title)
         content << "\n\n"
         content << _("TODO: SLIDE DESCRIPTION")
@@ -364,12 +586,12 @@ EOD
         content << "\n\n"
         content << generator.heading(3, _("Install"))
         content << "\n\n"
-        install_command = "gem install #{@slide_conf.gem_name}"
+        install_command = "gem install #{@data.slide_conf.gem_name}"
         content << generator.preformatted_line(install_command)
         content << "\n\n"
         content << generator.heading(3, _("Show"))
         content << "\n\n"
-        show_command = "rabbit #{@slide_conf.gem_name}.gem"
+        show_command = "rabbit #{@data.slide_conf.gem_name}.gem"
         content << generator.preformatted_line(show_command)
         content << "\n\n"
       end
@@ -386,7 +608,7 @@ Rabbit::Task::Slide.new do |task|
   spec = task.spec
   # spec.files += Dir.glob("doc/**/*.*")
   # spec.files -= Dir.glob("private/**/*.*")
-  # spec.add_runtime_dependency("YOUR THEME")
+  # spec.add_runtime_dependency("rabbit-theme-YOUR-THEME")
 end
 
 desc "Tag #{spec.version}"
@@ -407,11 +629,11 @@ end
       end
 
       def slide_path
-        "#{@slide_conf.base_name}.#{slide_source_extension}"
+        "#{@data.slide_conf.base_name}.#{slide_source_extension}"
       end
 
       def slide_source_extension
-        case @author_conf.markup_language
+        case @data.author_conf.markup_language
         when :rd
           "rab"
         when :hiki
@@ -424,7 +646,7 @@ end
       end
 
       def readme_extension
-        case @author_conf.markup_language
+        case @data.author_conf.markup_language
         when :rd
           "rd"
         when :hiki
@@ -437,7 +659,7 @@ end
       end
 
       def slide_source
-        generator = Rabbit::SourceGenerator.find(@author_conf.markup_language)
+        generator = Rabbit::SourceGenerator.find(@data.markup_language)
         return nil if generator.nil?
 
         source = ""
@@ -455,15 +677,36 @@ end
       end
 
       def slide_source_metadata(source, generator)
-        presentation_date = @slide_conf.presentation_date
+        presentation_date = @data.slide_conf.presentation_date
+        presentation_date_default = Time.now
+        allotted_time_default = "5m"
+        allotted_time =
+          Utils.ensure_time(@data.allotted_time || allotted_time_default)
+        start_time = @data.slide_conf.presentation_start_time
+        end_time = @data.slide_conf.presentation_end_time
+        if presentation_date
+          start_time ||= presentation_date
+          end_time ||= presentation_date + allotted_time
+          presentation_date = presentation_date.strftime("%Y-%m-%d")
+        end
+        start_time = start_time.iso8601 if start_time
+        end_time = end_time.iso8601 if end_time
+        start_time_default = presentation_date_default
+        end_time_default = start_time_default + allotted_time
         slide_metadata = [
-          ["subtitle",       nil,                _("SUBTITLE")],
-          ["author",         @author_conf.name,  _("AUTHOR")],
-          ["institution",    nil,                _("INSTITUTION")],
-          ["content-source", nil,                _("EVENT NAME")],
-          ["date",           presentation_date,  Time.now.strftime("%Y-%m-%d")],
-          ["allotted-time",  @allotted_time,     "5m"],
-          ["theme",          nil,                "default"],
+          ["subtitle",       nil,                     _("SUBTITLE")],
+          ["author",         @data.author_conf.name,  _("AUTHOR")],
+          ["institution",    nil,                     _("INSTITUTION")],
+          ["content-source", nil,                     _("EVENT NAME")],
+          [
+            "date",
+            presentation_date,
+            presentation_date_default.strftime("%Y-%m-%d"),
+          ],
+          ["allotted-time",  @data.allotted_time, "5m"],
+          ["start-time",     start_time,          start_time_default.iso8601],
+          ["end-time",       end_time,            end_time_default.iso8601],
+          ["theme",          nil,                 "default"],
         ]
         slide_metadata.each do |key, value, default_value|
           item = generator.definition_list_item(key, value || default_value)
@@ -501,7 +744,7 @@ end
         when "change"
           "."
         else
-          @slide_conf.id
+          @data.slide_conf.id
         end
       end
 
