@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2014  Kouhei Sutou <kou@cozmixng.org>
+# Copyright (C) 2004-2024  Sutou Kouhei <kou@cozmixng.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -142,7 +142,6 @@ module Rabbit
           options.migemo_dictionary_name = "migemo-dict"
           options.use_gl = false
           options.show_native_window_id = false
-          options.keep_above = false
           options.source_filename = nil
 
 
@@ -587,14 +586,6 @@ module Rabbit
             options.use_gl = bool
           end
 
-          parser.category _("Display")
-
-          parser.on("--[no-]keep-above",
-                    _("Specify whether to keep above window."),
-                    "(#{options.keep_above})") do |bool|
-            options.keep_above = bool
-          end
-
           parser.category _("Others")
 
           parser.on("--check-syntax",
@@ -645,14 +636,10 @@ module Rabbit
       end
 
       def choose_source_file_by_dialog
-        dialog = Gtk::FileChooserDialog.new(_("Choose a Rabbit source file"),
-                                            nil,
-                                            Gtk::FileChooser::ACTION_OPEN,
-                                            nil,
-                                            [Gtk::Stock::CANCEL,
-                                             Gtk::Dialog::RESPONSE_CANCEL],
-                                            [Gtk::Stock::OPEN,
-                                             Gtk::Dialog::RESPONSE_ACCEPT])
+        dialog = Gtk::FileChooserDialog.new(:title => _("Choose a Rabbit source file"),
+                                            :action => :open,
+                                            :buttons => [[Gtk::Stock::CANCEL, :cancel],
+                                                        [Gtk::Stock::OPEN, :accept]])
         dialog.current_folder = @options.base if @options.base
         add_source_dialog_filter(dialog, "Rabbit files", "*.rab")
         add_source_dialog_filter(dialog, "RD files", "*.rd")
@@ -661,7 +648,7 @@ module Rabbit
         add_source_dialog_filter(dialog, "Markdown files", "*.md")
         add_source_dialog_filter(dialog, "All files", "*")
         file_name = nil
-        if dialog.run == Gtk::Dialog::RESPONSE_ACCEPT
+        if dialog.run == Gtk::ResponseType::ACCEPT
           file_name = dialog.filename
           dialog.destroy
         end
@@ -727,9 +714,9 @@ module Rabbit
         target.draw_scaled_image = @options.draw_scaled_image
       end
 
-      def setup_size(target)
-        target.width = @options.width
-        target.height = @options.height
+      def setup_base_size(target)
+        target.base_width = @options.width
+        target.base_height = @options.height
       end
 
       def setup_paper_size(target)
@@ -811,8 +798,8 @@ module Rabbit
 
       def do_print
         source = make_source
-        renderer = Renderer.printable_renderer(@options.slides_per_page)
-        canvas = make_canvas(renderer)
+        canvas = make_canvas(Renderer::Printer)
+        setup_base_size(canvas)
         setup_paper_size(canvas)
         setup_print_info(canvas)
         setup_3d_info(canvas)
@@ -829,7 +816,7 @@ module Rabbit
       def do_save_as_image
         source = make_source
         canvas = make_canvas(Renderer::Offscreen)
-        setup_size(canvas)
+        setup_base_size(canvas)
         setup_image_info(canvas)
         setup_print_info(canvas)
         setup_paper_size(canvas)
@@ -851,10 +838,14 @@ module Rabbit
         Renderer::Display.init(display_init_options)
 
         source = make_source
-        canvas = make_canvas(Renderer::Display)
+        if ENV["RABBIT_RENDERER"] == "screen"
+          canvas = make_canvas(Renderer::Screen)
+        else
+          canvas = make_canvas(Renderer::Display)
+        end
         frame = Frame.new(@logger, canvas)
         frame.geometry = @options.geometry
-        frame.force_keep_above = @options.keep_above
+        setup_base_size(canvas)
         setup_paper_size(canvas)
         setup_image_info(canvas)
         setup_print_info(canvas)
@@ -862,9 +853,11 @@ module Rabbit
         setup_3d_info(canvas)
         frame.init_gui(@options.width, @options.height, true)
         frame.fullscreen if @options.full_screen
-        native_window = frame.window.window
-        if @options.show_native_window_id and native_window.respond_to?(:xid)
-          @logger.info(_("Window ID: %d") % native_window.xid)
+        if @options.show_native_window_id
+          native_surface = frame.window.surface
+          if native_surface.respond_to?(:xid)
+            @logger.info(_("Window ID: %d") % native_surface.xid)
+          end
         end
         apply_theme_if_need(frame)
         parse(frame, source, !Utils.windows?)
@@ -881,8 +874,7 @@ module Rabbit
 
       def do_check_syntax
         source = make_source
-        renderer = Renderer.printable_renderer(1)
-        canvas = make_canvas(renderer)
+        canvas = make_canvas(Renderer::Printer)
         exception = nil
         begin
           canvas.parse(source) do |_exception|
