@@ -1,15 +1,38 @@
+# Copyright (C) 2004-2024  Sutou Kouhei <kou@cozmixng.org>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+
 require "forwardable"
 require "rabbit/gtk"
 require "rexml/text"
 
+begin
+  case ENV["RABBIT_GTK"]
+  when "4"
+    require "vte4"
+  else
+    require "vte3"
+  end
+rescue LoadError
+end
+
 require "rabbit/rabbit"
 require "rabbit/utils"
 
 module Rabbit
-
   class Frame
-
-    include ScreenInfo
     extend Forwardable
 
     def_delegators(:@window, :icon, :icon=, :set_icon)
@@ -25,13 +48,15 @@ module Rabbit
     def_delegators(:@canvas, :rss_base_uri=, :use_gl?, :use_gl=)
 
     attr_reader :window, :logger
-    attr_accessor :geometry, :force_keep_above
+    attr_accessor :geometry
 
     def initialize(logger, canvas)
       @logger = logger
       @canvas = canvas
       @geometry = nil
-      @force_keep_above = nil
+      @notebook = nil
+      @terminal = nil
+      @running = true
     end
 
     def destroyed?
@@ -39,6 +64,7 @@ module Rabbit
     end
 
     def quit
+      @running = false
       @window.destroy unless destroyed?
       @window = nil
       true
@@ -88,11 +114,8 @@ module Rabbit
       init_window(width, height, window_type)
       @fullscreen = false
       @main_window = main_window
-      if @main_window
-        @window.keep_above = @force_keep_above unless @force_keep_above.nil?
-      else
-        @window.keep_above = true
-      end
+      @terminal.show if @terminal
+      @notebook.show if @notebook
       @window.show
       @canvas.post_init_gui
     end
@@ -105,6 +128,21 @@ module Rabbit
       true
     end
 
+    def toggle_terminal
+      return if @terminal.nil?
+      terminal_page = @notebook.page_num(@terminal)
+      if @notebook.current_page == terminal_page
+        @notebook.current_page = 0
+      else
+        @notebook.current_page = terminal_page
+      end
+    end
+
+    def in_terminal?
+      return false if @terminal.nil?
+      @notebook.current_page == @notebook.page_num(@terminal)
+    end
+
     private
     def init_window(width, height, window_type=nil)
       window_type ||= :toplevel
@@ -112,9 +150,29 @@ module Rabbit
       @window.set_default_size(width, height)
       @window.parse_geometry(@geometry) if @geometry
       @window.set_app_paintable(true)
+      if defined?(Vte::Terminal)
+        init_notebook
+      end
       set_window_signal
       setup_dnd
-      @canvas.attach_to(self, @window)
+      @canvas.attach_to(self, @window, @notebook)
+      if defined?(Vte::Terminal)
+        init_terminal
+      end
+    end
+
+    def init_notebook
+      @notebook = Gtk::Notebook.new
+      @notebook.show_tabs = false
+      provider = Gtk::CssProvider.new
+      provider.load(data: <<-CSS)
+        notebook {
+          border-width: 0px;
+        }
+      CSS
+      @notebook.style_context.add_provider(provider,
+                                           Gtk::StyleProvider::PRIORITY_USER)
+      @window.add(@notebook)
     end
 
     def set_window_signal
@@ -122,24 +180,13 @@ module Rabbit
       set_window_signal_destroy
     end
 
-    def update_keep_above(keep_above)
-      if @main_window
-        keep_above = @force_keep_above unless @force_keep_above.nil?
-        @window.keep_above = keep_above
-      else
-        @window.keep_above = true
-      end
-    end
-
     def set_window_signal_window_state_event
       @window.signal_connect("window_state_event") do |widget, event|
         if event.changed_mask.fullscreen?
           @fullscreen = event.new_window_state.fullscreen?
           if @fullscreen
-            @window.keep_above = true
             @canvas.fullscreened
           else
-            update_keep_above(false)
             @canvas.unfullscreened
           end
         elsif event.changed_mask.iconified?
@@ -177,6 +224,47 @@ module Rabbit
         true
       end
     end
+
+    def init_terminal
+      @terminal = Vte::Terminal.new
+      # TODO: Support theme
+      terminal_font_description = ENV["RABBIT_TERMINAL_FONT_DESCRIPTION"]
+      if terminal_font_description
+        @terminal.font_desc =
+          Pango::FontDescription.new(terminal_font_description)
+      end
+      terminal_color_foreground = ENV["RABBIT_TERMINAL_COLOR_FOREGROUND"]
+      if terminal_color_foreground
+        @terminal.color_foreground = terminal_color_foreground
+      end
+      terminal_color_background = ENV["RABBIT_TERMINAL_COLOR_BACKGROUND"]
+      if terminal_color_background
+        @terminal.color_background = terminal_color_background
+      end
+      @terminal.enable_sixel = true if @terminal.respond_to?(:enable_sixel=)
+      @notebook.add(@terminal)
+      pid = nil
+      in_terminal = false
+      @notebook.signal_connect(:switch_page) do |_, page,|
+        if page == @terminal
+          if @running
+            pid = @terminal.spawn if pid.nil?
+            @canvas.pre_terminal unless in_terminal
+            in_terminal = true
+          end
+        else
+          @canvas.post_terminal if in_terminal
+          in_terminal = false
+        end
+      end
+      @terminal.signal_connect(:child_exited) do
+        pid = nil
+        terminal_page = @notebook.page_num(@terminal)
+        if @notebook.current_page == terminal_page
+          @canvas.activate("ToggleTerminal")
+        end
+      end
+    end
   end
 
   class NullFrame
@@ -191,7 +279,6 @@ module Rabbit
     def_null_methods(:icon, :icon=, :set_icon)
     def_null_methods(:icon_list, :icon_list=, :set_icon_list)
     def_null_methods(:update_title, :geometry, :geometry=)
-    def_null_methods(:force_keep_above, :force_keep_above=)
 
     def_null_methods(:fullscreen?, :quit)
 
@@ -202,10 +289,16 @@ module Rabbit
     def iconify_available?
       false
     end
+
+    def toggle_terminal
+    end
+
+    def in_terminal?
+      false
+    end
   end
 
   class EmbedFrame < Frame
-
     def update_title(new_title)
     end
 
@@ -217,6 +310,13 @@ module Rabbit
       false
     end
 
+    def toggle_terminal
+    end
+
+    def in_terminal?
+      false
+    end
+
     def init_gui(width, height, main_window, window_type=nil)
       @window = Gtk::EventBox.new
       @window.set_size_request(width, height)
