@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2017  Kouhei Sutou <kou@cozmixng.org>
+# Copyright (C) 2004-2022  Sutou Kouhei <kou@cozmixng.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,13 +22,13 @@ require "rabbit/parser/rd/rt/rt2rabbit-lib"
 require "rabbit/parser/rd/ext/base"
 require "rabbit/parser/rd/ext/image"
 require "rabbit/parser/rd/ext/video"
-require "rabbit/parser/ext/enscript"
-require "rabbit/parser/ext/tex"
 require "rabbit/parser/ext/aafigure"
 require "rabbit/parser/ext/blockdiag"
+require "rabbit/parser/ext/charty"
 require "rabbit/parser/ext/coderay"
-require "rabbit/parser/ext/emacs"
+require "rabbit/parser/ext/enscript"
 require "rabbit/parser/ext/rouge"
+require "rabbit/parser/ext/tex"
 
 module Rabbit
   module Parser
@@ -54,26 +54,7 @@ module Rabbit
             return nil unless /\A(?:image|img)\z/i =~ label
             src, prop = parse_source(source)
             return nil if prop["src"].nil?
-
-            if prop["align"] == "right"
-              body = visitor.current_body
-              if body["background-image"]
-                raise ParseError,
-                      _("multiple 'align = right' isn't supported.")
-              end
-              prop.each do |name, value|
-                name = name.gsub(/_/, "-")
-                if name == "src"
-                  property_name = "background-image"
-                else
-                  property_name = "background-image-#{name}"
-                end
-                body[property_name] = value
-              end
-              :no_element
-            else
-              make_image(visitor, prop["src"], prop)
-            end
+            make_image(visitor, prop["src"], prop, body: visitor.current_body)
           end
 
           def ext_block_verb_video(label, source, content, visitor)
@@ -86,7 +67,7 @@ module Rabbit
 
           def ext_block_verb_enscript(label, source, content, visitor)
             return nil unless /\Aenscript (\w+)\z/i =~ label
-            lang = $1.downcase.untaint
+            lang = $1.downcase
 
             src, prop = parse_source(source)
             logger = visitor.logger
@@ -100,36 +81,36 @@ module Rabbit
 
           def ext_block_verb_LaTeX(label, source, content, visitor)
             return nil unless /\ALaTeX\z/i =~ label
-            make_image_from_file(source, visitor) do |src_file_path, prop|
-              Parser::Ext::TeX.make_image_by_LaTeX(src_file_path, prop, visitor)
+            make_image_from_file(source, visitor) do |src_file, prop|
+              Parser::Ext::TeX.make_image_by_LaTeX(src_file.path, prop, visitor)
             end
           end
 
           def ext_block_verb_mimeTeX(label, source, content, visitor)
             return nil unless /\AmimeTeX\z/i =~ label
-            make_image_from_file(source, visitor) do |src_file_path, prop|
-              Parser::Ext::TeX.make_image_by_mimeTeX(src_file_path, prop,
+            make_image_from_file(source, visitor) do |src_file, prop|
+              Parser::Ext::TeX.make_image_by_mimeTeX(src_file.path, prop,
                                                      visitor)
             end
           end
 
           def ext_block_verb_aafigure(label, source, content, visitor)
             return nil unless /\Aaafigure\z/i =~ label
-            make_image_from_file(source, visitor) do |src_file_path, prop|
-              Parser::Ext::AAFigure.make_image(src_file_path, prop, visitor)
+            make_image_from_file(source, visitor) do |src_file, prop|
+              Parser::Ext::AAFigure.make_image(src_file.path, prop, visitor)
             end
           end
 
           def ext_block_verb_blockdiag(label, source, content, visitor)
             return nil unless /\Ablockdiag\z/i =~ label
-            make_image_from_file(source, visitor) do |src_file_path, prop|
-              Parser::Ext::BlockDiag.make_image(src_file_path, prop, visitor)
+            make_image_from_file(source, visitor) do |src_file, prop|
+              Parser::Ext::BlockDiag.make_image(src_file.path, prop, visitor)
             end
           end
 
           def ext_block_verb_coderay(label, source, content, visitor)
             return nil unless /\Acoderay (\w+)\z/i =~ label
-            lang = $1.downcase.untaint
+            lang = $1.downcase
 
             src, prop = parse_source(source)
             logger = visitor.logger
@@ -138,17 +119,6 @@ module Rabbit
             result || default_ext_block_verbatim(label, src, src, visitor)
           end
 
-          def ext_block_verb_emacs(label, source, content, visitor)
-            return nil unless /\Aemacs(?:\s+(.+))?\z/i =~ label
-            mode_line = $1.untaint
-
-            src, prop = parse_source(source)
-            logger = visitor.logger
-
-            result = Parser::Ext::Emacs.highlight(src, logger, mode_line)
-            result || default_ext_block_verbatim(label, src, src, visitor)
-          end
-
           def ext_block_verb_rt(label, source, content, visitor)
             return nil unless /\Art\z/i =~ label
             unless defined?(RT2RabbitVisitor)
@@ -193,7 +163,7 @@ module Rabbit
 
           def ext_block_verb_rouge(label, source, content, visitor)
             return nil unless /\Arouge (\w+)\z/i =~ label
-            lang = $1.downcase.untaint
+            lang = $1.downcase
 
             src, prop = parse_source(source)
             logger = visitor.logger
@@ -201,6 +171,22 @@ module Rabbit
             result = Parser::Ext::Rouge.highlight(lang, src, logger)
             result || default_ext_block_verbatim(label, src, src, visitor)
           end
+
+          def ext_block_verb_charty(label, source, content, visitor)
+            return nil unless /\Acharty\z/i =~ label
+            make_image_from_file(source, visitor) do |src_file, prop|
+              Parser::Ext::Charty.make_image(src_file.path, prop, visitor)
+            end
+          end
+
+          def ext_block_verb_mermaid(label, source, content, visitor)
+            return nil unless /\Amermaid\z/i =~ label
+            make_image_from_file(source,
+                                 visitor,
+                                 extension: ".mmd") do |src_file, prop|
+              src_file
+            end
+          end
         end
       end
     end
