@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2017  Kouhei Sutou <kou@cozmixng.org>
+# Copyright (C) 2007-2022  Sutou Kouhei <kou@cozmixng.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@ require "rabbit/parser/ext/tex"
 require "rabbit/parser/ext/aafigure"
 require "rabbit/parser/ext/blockdiag"
 require "rabbit/parser/ext/coderay"
-require "rabbit/parser/ext/emacs"
 require "rabbit/parser/ext/entity"
 require "rabbit/parser/ext/rouge"
 
@@ -489,9 +488,9 @@ module Rabbit
 
           def LaTeX(source, props={})
             args = [@output.canvas, source]
-            Ext::Image.make_image_from_file(*args) do |src_file_path|
+            Ext::Image.make_image_from_file(*args) do |src_file|
               props = Utils.stringify_hash_key(props)
-              args = [src_file_path, props, @output.canvas]
+              args = [src_file.path, props, @output.canvas]
               [Ext::TeX.make_image_by_LaTeX(*args), props]
             end
           end
@@ -499,9 +498,9 @@ module Rabbit
 
           def mimeTeX(source, props={})
             args = [@output.canvas, source]
-            Ext::Image.make_image_from_file(*args) do |src_file_path|
+            Ext::Image.make_image_from_file(*args) do |src_file|
               props = Utils.stringify_hash_key(props)
-              args = [src_file_path, props, @output.canvas]
+              args = [src_file.path, props, @output.canvas]
               [Ext::TeX.make_image_by_mimeTeX(*args), props]
             end
           end
@@ -509,18 +508,18 @@ module Rabbit
 
           def aafigure(source, props={})
             args = [@output.canvas, source]
-            Ext::Image.make_image_from_file(*args) do |src_file_path|
+            Ext::Image.make_image_from_file(*args) do |src_file|
               props = Utils.stringify_hash_key(props)
-              args = [src_file_path, props, @output.canvas]
+              args = [src_file.path, props, @output.canvas]
               [Ext::AAFigure.make_image(*args), props]
             end
           end
 
           def blockdiag(source, props={})
             args = [@output.canvas, source]
-            Ext::Image.make_image_from_file(*args) do |src_file_path|
+            Ext::Image.make_image_from_file(*args) do |src_file|
               props = Utils.stringify_hash_key(props)
-              args = [src_file_path, props, @output.canvas]
+              args = [src_file.path, props, @output.canvas]
               [Ext::BlockDiag.make_image(*args), props]
             end
           end
@@ -530,11 +529,6 @@ module Rabbit
             Ext::CodeRay.highlight(lang, source, logger)
           end
 
-          def emacs(source, mode_line=nil)
-            logger = @output.canvas.logger
-            Ext::Emacs.highlight(source, logger, mode_line)
-          end
-
           def rouge(lang, source)
             logger = @output.canvas.logger
             Ext::Rouge.highlight(lang, source, logger)
