@@ -1,10 +1,25 @@
+# Copyright (C) 2004-2024  Sutou Kouhei <kou@cozmixng.org>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+
 require 'rabbit/gtk'
 
 require "rabbit/utils"
 require "rabbit/renderer/engine"
 require "rabbit/renderer/display/base"
 require 'rabbit/renderer/display/cursor'
-require 'rabbit/renderer/display/gl'
 
 module Rabbit
   module Renderer
@@ -13,7 +28,6 @@ module Rabbit
         include Base
 
         include Cursor
-        include GL
 
         attr_accessor :filename
         def initialize(canvas)
@@ -24,12 +38,17 @@ module Rabbit
         end
 
         def attach_to(window, container=nil)
-          init_gl(@area)
           super
+
+          set_configure_event
         end
 
         def detach
-          finalize_gl
+          if !@window.destroyed? and @configure_signal_id
+            @window.signal_handler_disconnect(@configure_signal_id)
+            @configure_signal_id = nil
+          end
+
           super
         end
 
@@ -74,6 +93,7 @@ module Rabbit
 
         def post_parse
           clear_compiled_slides
+          update_cursor(:blank, true)
         end
 
         def pre_toggle_index_mode
@@ -107,11 +127,6 @@ module Rabbit
         end
 
         private
-        def init_dpi
-          @x_dpi = ScreenInfo.screen_x_resolution
-          @y_dpi = ScreenInfo.screen_y_resolution
-        end
-
         def init_drawing_area
           @area = Gtk::DrawingArea.new
           @area.can_focus = true
@@ -131,25 +146,24 @@ module Rabbit
         end
 
         def mapped(widget)
-          set_drawable(widget.window)
-          prepare_renderer(@drawable)
+          set_surface(widget.native.surface)
         end
 
         def set_draw
-          stop_events = false
-          if @area.class.signals.include?("draw")
+          if @area.respond_to?(:set_draw_func)
+            @area.set_draw_func do |area, context|
+              surface = area.native.surface
+              update_size(surface.width, surface.height)
+              init_context(context)
+              draw(area)
+              finish_renderer
+            end
+          else
             @area.signal_connect("draw") do |widget, context|
               init_context(context)
               draw(widget)
               finish_renderer
-              stop_events
-            end
-          else
-            @area.signal_connect("expose_event") do |widget, event|
-              init_renderer(@drawable)
-              draw(widget)
-              finish_renderer
-              stop_events
+              Gdk::Event::PROPAGATE
             end
           end
         end
@@ -170,6 +184,20 @@ module Rabbit
           end
         end
 
+        def set_configure_event
+          id = @window.signal_connect("configure_event") do |widget, event|
+            configured(event.x, event.y, event.width, event.height)
+            false
+          end
+          @configure_signal_id = id
+        end
+
+        def configured(x, y, w, h)
+          @real_width = @surface.width
+          @real_height = @surface.height
+          @size_dirty = true
+        end
+
         def set_configure_event_after
           prev_x = prev_y = prev_width = prev_height = nil
           @area.signal_connect_after("configure_event") do |widget, event|
@@ -190,7 +218,8 @@ module Rabbit
         end
 
         def configured_after(widget, event)
-          reload_theme if @drawable
+          update_size(event.width, event.height)
+          reload_theme if @surface
         end
 
         def reload_theme(&callback)
@@ -216,19 +245,19 @@ module Rabbit
         end
 
         def grab
-          Gtk.grab_add(@area)
+          @area.grab_add
           Gdk.pointer_grab(@area.window, false,
                            Gdk::EventMask::BUTTON_PRESS_MASK |
                            Gdk::EventMask::BUTTON_RELEASE_MASK |
                            Gdk::EventMask::SCROLL_MASK |
                            Gdk::EventMask::POINTER_MOTION_MASK,
                            nil, nil,
-                           Gdk::Event::CURRENT_TIME)
+                           Gdk::CURRENT_TIME)
         end
 
         def ungrab
-          Gtk.grab_remove(@area)
-          Gdk.pointer_ungrab(Gdk::Event::CURRENT_TIME)
+          @area.grab_remove
+          Gdk.pointer_ungrab(Gdk::CURRENT_TIME)
         end
 
         def pointer
