@@ -1,3 +1,21 @@
+# Copyright (C) 2006-2018  Kouhei Sutou <kou@cozmixng.org>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+
+require "rabbit/pango-markup"
+
 require "rabbit/renderer/color"
 
 module Rabbit
@@ -173,51 +191,6 @@ module Rabbit
         [params["pole_width"] + params["flag_width"], pole_height]
       end
 
-      def draw_cube(filled, x, y, z, size, color=nil)
-        not_support_method("draw_cube")
-      end
-
-      def draw_sphere(filled, x, y, z, radius, slices, stacks, color=nil)
-        not_support_method("draw_sphere")
-      end
-
-      def draw_cone(filled, x, y, z, base, height, slices, stacks, color=nil)
-        not_support_method("draw_cone")
-      end
-
-      def draw_torus(filled, x, y, z, inner_radius, outer_radius,
-                     n_sides, rings, color=nil)
-        not_support_method("draw_torus")
-      end
-
-      def draw_tetrahedron(filled, x, y, z, color=nil)
-        not_support_method("draw_tetrahedron")
-      end
-
-      def draw_octahedron(filled, x, y, z, color=nil)
-        not_support_method("draw_octahedron")
-      end
-
-      def draw_dodecahedron(filled, x, y, z, color=nil)
-        not_support_method("draw_dodecahedron")
-      end
-
-      def draw_icosahedron(filled, x, y, z, color=nil)
-        not_support_method("draw_icosahedron")
-      end
-
-      def draw_teapot(filled, x, y, z, scale, color=nil)
-        not_support_method("draw_teapot")
-      end
-
-      def gl_compile(id)
-        not_support_method("gl_compile")
-      end
-
-      def gl_call_list(id, x, y, z, color=nil)
-        not_support_method("gl_call_list")
-      end
-
       def gl_supported?
         false
       end
@@ -230,11 +203,6 @@ module Rabbit
         5.0
       end
 
-      def new_list_id
-        @list_id += 1
-        @list_id
-      end
-
       private
       def get_line_width(params, default=nil)
         params[:line_width] || default
@@ -242,6 +210,42 @@ module Rabbit
 
       def set_font_resolution(context)
       end
+
+      def setup_flag_params(pole_height, default_flag_width_ratio, params)
+        params = params.dup
+
+        text = params["text"]
+        text_attrs = params["text_attributes"] || {}
+        if text
+          markupped_text = PangoMarkup.new("span", text_attrs, text)
+          layout = make_layout(markupped_text.to_s)
+          text_width, text_height = layout.pixel_size
+          params["layout"] = layout
+          params["text_width"] = text_width
+          params["text_height"] = text_height
+          flag_width_default = [
+            text_width * default_flag_width_ratio,
+            pole_height / 2
+          ].max
+          flag_height_default = [text_height, flag_width_default].max
+        else
+          params["layout"] = nil
+          flag_width_default = flag_height_default = nil
+        end
+
+        params["pole_width"] = params["pole_width"] || 2
+        params["pole_color"] ||= "black"
+        flag_height = params["flag_height"] ||
+          flag_height_default || pole_height / 2
+        flag_height = [flag_height, pole_height].min
+        params["flag_height"] = flag_height
+        params["flag_width"] ||= flag_width_default || flag_height
+        params["flag_color"] ||= "red"
+        params["flag_frame_width"] ||= params["pole_width"]
+        params["flag_frame_color"] ||= params["pole_color"]
+
+        params
+      end
     end
   end
 end
