@@ -1,14 +1,8 @@
 @default_foreground = "#000000"
 @default_background = "#ffffff"
 
-@slide_number_uninstall = true
-
 @default_item1_mark_color = "#a00000"
 @default_item1_mark_type = "circle"
 @default_item2_mark_color = "#a00000"
 @default_item2_mark_type = "circle"
 include_theme("default")
-
-@image_slide_number_show_text = true
-include_theme("image-slide-number")
-include_theme("image-timer")
