@@ -24,7 +24,8 @@ fill_color = "#ffffff4d"
 @table_head_fill_color = dark_color
 @table_header_font_props[:color] = "white"
 
-@slide_number_uninstall = true
+@image_slide_number_start_flag_color = dark_color
+@image_slide_number_goal_flag_color = dark_color
 
 include_theme("default")
 
@@ -45,11 +46,3 @@ background_image = "rubykaigi2011-background-white.jpg"
 @slide_background_image = background_image
 include_theme("title-slide-background-image")
 include_theme("slide-background-image")
-
-if canvas.allotted_time
-  include_theme("image-slide-number")
-  @image_slide_number_show_text = true
-  @image_slide_number_start_flag_color = dark_color
-  @image_slide_number_goal_flag_color = dark_color
-  include_theme("image-timer")
-end
