@@ -1,3 +1,3 @@
 format = _("%s is deprecated. Use %s instead.")
-warn(format % ["slide-background", "slide-background-image"])
+warning(format % ["slide-background", "slide-background-image"])
 include_theme("slide-background-image")
