@@ -1,4 +1,4 @@
 @category = N_("Toolkit")
 @title = N_("Tag")
 @abstract = N_("Toolkit to apply custom tag")
-@description = N_("Applyes custom tag style.")
+@description = N_("Applies custom tag style.")
