@@ -1,4 +1,4 @@
-# Copyright (C) 2004-2015  Kouhei Sutou <kou@cozmixng.org>
+# Copyright (C) 2004-2024  Sutou Kouhei <kou@cozmixng.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,7 +15,6 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 require 'tempfile'
-require 'nkf'
 
 module Rabbit
   module Utils
@@ -352,9 +351,32 @@ module Rabbit
 
   module SystemRunner
     module_function
-    def run(cmd, *args)
+    def run(cmd, *args, progress: nil)
       begin
-        system(cmd, *args)
+        IO.pipe do |input, output|
+          pid = spawn(cmd, *args, out: output)
+          output.close
+          begin
+            loop do
+              readables, = IO.select([input], nil, nil, 0.1)
+              if readables
+                readable = readables[0]
+                begin
+                  $stdout.print(readable.read_nonblock(4096))
+                rescue EOFError
+                  break
+                else
+                  break if readable.eof?
+                end
+              else
+                progress.call if progress
+              end
+            end
+            true
+          ensure
+            Process.waitpid(pid)
+          end
+        end
       rescue SystemCallError
         yield($!) if block_given?
         false
@@ -362,45 +384,6 @@ module Rabbit
     end
   end
 
-  module ScreenInfo
-    module_function
-    def default_screen
-      Gdk::Screen.default
-    end
-
-    def screen_width
-      default_screen.width
-    end
-
-    def screen_width_mm
-      default_screen.width_mm
-    end
-
-    def screen_height
-      default_screen.height
-    end
-
-    def screen_height_mm
-      default_screen.height_mm
-    end
-
-    def screen_x_resolution
-      screen_width / mm_to_inch(screen_width_mm)
-    end
-
-    def screen_y_resolution
-      screen_height / mm_to_inch(screen_height_mm)
-    end
-
-    def screen_depth
-      default_screen.system_visual.depth
-    end
-
-    def mm_to_inch(mm)
-      mm / 25.4
-    end
-  end
-
   module HTML
     module_function
     def a_link(start_a, label, label_only)
@@ -468,19 +451,19 @@ module Rabbit
     end
 
     def to_utf8(str)
-      NKF.nkf("-w", str)
+      str.encode(Encoding::UTF_8)
     end
 
     def to_eucjp(str)
-      NKF.nkf("-e", str)
+      str.encode(Encoding::EUCJP)
     end
 
     def to_utf8_from_eucjp(str)
-      NKF.nkf("-wE", str)
+      str.encode(Encoding::UTF_8, Encoding::EUCJP)
     end
 
     def to_eucjp_from_utf8(str)
-      NKF.nkf("-eW", str)
+      str.encode(Encoding::EUCJP, Encoding::UTF_8)
     end
   end
 
