@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2017  Kouhei Sutou <kou@cozmixng.org>
+# Copyright (C) 2014-2024  Sutou Kouhei <kou@cozmixng.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,16 +14,14 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-require "rabbit-test-utils"
-
 require "rabbit/logger"
 require "rabbit/slide"
 require "rabbit/source/memory"
 require "rabbit/parser/markdown"
 
 class RabbitParserMarkdownTest < Test::Unit::TestCase
-  include RabbitTestUtils::Fixture
-  include RabbitTestUtils::Parser
+  include Helper::Fixture
+  include Helper::Parser
 
   private
   def parse(markdown)
@@ -46,7 +44,7 @@ class RabbitParserMarkdownTest < Test::Unit::TestCase
                        ],
                      ],
                    ],
-                   parse("# Title"))
+                   parse(+"# Title"))
     end
   end
 
@@ -289,7 +287,8 @@ class RabbitParserMarkdownTest < Test::Unit::TestCase
 
         def test_unsupported
           image_path = fixture_path("image/png/lavie.png")
-          message = _("multiple ![alt]{image} in a paragraph isn't supported.")
+          format = _("![alt]{image} and other contents in a paragraph isn't supported: %{types}")
+          message = format % {types: [:text, :img]}
           assert_raise(Rabbit::ParseError.new(message)) do
             parse(<<-MARKDOWN)
 a ![](#{image_path})
