@@ -1,8 +1,18 @@
-# -*- coding: utf-8 -*-
-
-require 'fileutils'
-
-require "rabbit-test-utils"
+# Copyright (C) 2004-2024  Kouhei Sutou <kou@cozmixng.org>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 require "rabbit/source"
 require "rabbit/logger"
@@ -25,7 +35,7 @@ class RabbitSourceTest < Test::Unit::TestCase
 
     class ReadTest < self
       def test_binary
-        pdf_header = "%PDF-1.5\n%\xb5\xed\xae\xfb\n"
+        pdf_header = +"%PDF-1.5\n%\xb5\xed\xae\xfb\n"
         pdf_header.force_encoding("ASCII-8BIT")
         assert_equal(pdf_header, read(pdf_header))
       end
